import { jsx as _jsx } from "react/jsx-runtime";
import { createContext, useContext, useEffect, useState } from "react";
import { useRerender } from "./util/useRerender";
/**
 * Create various React-specific hooks and components for your
 * Miniplex ECS instance.
 *
 * @param world An instance of a Miniplex ECS to use.
 */
export function createReactIntegration(world) {
    function useArchetype(...names) {
        const rerender = useRerender();
        const archetype = world.createArchetype(...names);
        useEffect(() => {
            world.listeners.archetypeChanged.get(archetype).on(rerender);
            return () => world.listeners.archetypeChanged.get(archetype).off(rerender);
        }, [world]);
        return world.get(archetype);
    }
    const EntityContext = createContext(null);
    const Entity = ({ children }) => {
        const [entity] = useState(() => ({}));
        useEffect(() => {
            world.addEntity(entity);
            return () => world.removeEntity(entity);
        }, [entity]);
        return _jsx(EntityContext.Provider, { value: entity, children: children }, void 0);
    };
    function Component({ name, data }) {
        const entity = useContext(EntityContext);
        useEffect(() => {
            world.addComponent(entity, name, data);
            return () => world.removeComponent(entity, name);
        }, [entity, name, data]);
        return null;
    }
    return { useArchetype, Entity, Component };
}
